'use strict';

Object.defineProperty(exports, '__esModule', { value: true });

var _rollupPluginBabelHelpers = require('./_virtual/_rollupPluginBabelHelpers.js');
var path = require('path');
var fsExtra = require('fs-extra');
var graphql = require('graphql');
var graphqlConfig = require('graphql-config');
var graphqlConfigUtilities = require('graphql-config-utilities');
var graphqlToolUtilities = require('graphql-tool-utilities');
var validate = require('./validate.js');

function evaluateFixtures(_x, _x2) {
  return _evaluateFixtures.apply(this, arguments);
}

function _evaluateFixtures() {
  _evaluateFixtures = _rollupPluginBabelHelpers.asyncToGenerator(function* (fixturePaths, {
    cwd
  }) {
    const config = yield graphqlConfig.loadConfig({
      rootDir: path.resolve(cwd)
    });

    if (!config) {
      return [];
    }

    const projectASTCollection = yield Promise.all(graphqlConfigUtilities.getGraphQLProjects(config).map(getOperationsForProject));
    return runForEachFixture(fixturePaths, fixture => evaluateFixture(fixture, projectASTCollection));
  });
  return _evaluateFixtures.apply(this, arguments);
}

function getOperationsForProject(_x3) {
  return _getOperationsForProject.apply(this, arguments);
}

function _getOperationsForProject() {
  _getOperationsForProject = _rollupPluginBabelHelpers.asyncToGenerator(function* (projectConfig) {
    const operationPaths = yield graphqlConfigUtilities.getGraphQLProjectIncludedFilePaths(projectConfig);
    const operationSources = yield Promise.all(operationPaths.map(loadOperationSource));
    const document = graphql.concatAST(operationSources.map(source => {
      try {
        return graphql.parse(source);
      } catch (error) {
        throw new Error(`Error parsing '${source.name}':\n\n${error.message.replace(/Syntax Error.*?\(.*?\) /, '')}`);
      }
    }));
    let schema;

    try {
      schema = yield projectConfig.getSchema();
    } catch (error) {
      throw new Error(`Error parsing '${projectConfig.schema}':\n\n${error.message.replace(/Syntax Error.*?\(.*?\) /, '')}`);
    }

    return {
      ast: graphqlToolUtilities.compile(schema, document),
      config: projectConfig
    };
  });
  return _getOperationsForProject.apply(this, arguments);
}

function loadOperationSource(_x4) {
  return _loadOperationSource.apply(this, arguments);
}

function _loadOperationSource() {
  _loadOperationSource = _rollupPluginBabelHelpers.asyncToGenerator(function* (filePath) {
    const body = yield fsExtra.readFile(filePath, 'utf8');
    return new graphql.Source(body, filePath);
  });
  return _loadOperationSource.apply(this, arguments);
}

function runForEachFixture(fixturePaths, runner) {
  return Promise.all(fixturePaths.map( /*#__PURE__*/function () {
    var _ref = _rollupPluginBabelHelpers.asyncToGenerator(function* (fixturePath) {
      try {
        const fixture = yield fsExtra.readJSON(fixturePath);
        return _rollupPluginBabelHelpers.objectSpread2({
          fixturePath
        }, runner({
          path: fixturePath,
          content: fixture
        }));
      } catch (error) {
        return {
          fixturePath,
          scriptError: error,
          validationErrors: []
        };
      }
    });

    return function (_x5) {
      return _ref.apply(this, arguments);
    };
  }()));
}

function evaluateFixture(fixture, projectASTCollection) {
  try {
    const {
      operation,
      projectAST: {
        ast
      }
    } = validate.getOperationForFixture(fixture, projectASTCollection);
    return validate.validateFixture(fixture, ast, operation);
  } catch (error) {
    if (error instanceof validate.MissingOperationError || error instanceof validate.AmbiguousOperationNameError) {
      return {
        fixturePath: fixture.path,
        validationErrors: [error]
      };
    }

    throw error;
  }
}

exports.evaluateFixtures = evaluateFixtures;
