import { asyncToGenerator as _asyncToGenerator, objectSpread2 as _objectSpread2 } from './_virtual/_rollupPluginBabelHelpers.js';
import { resolve } from 'path';
import { readFile, readJSON } from 'fs-extra';
import { concatAST, parse, Source } from 'graphql';
import { loadConfig } from 'graphql-config';
import { getGraphQLProjects, getGraphQLProjectIncludedFilePaths } from 'graphql-config-utilities';
import { compile } from 'graphql-tool-utilities';
import { getOperationForFixture, validateFixture, MissingOperationError, AmbiguousOperationNameError } from './validate.mjs';

function evaluateFixtures(_x, _x2) {
  return _evaluateFixtures.apply(this, arguments);
}

function _evaluateFixtures() {
  _evaluateFixtures = _asyncToGenerator(function* (fixturePaths, {
    cwd
  }) {
    const config = yield loadConfig({
      rootDir: resolve(cwd)
    });

    if (!config) {
      return [];
    }

    const projectASTCollection = yield Promise.all(getGraphQLProjects(config).map(getOperationsForProject));
    return runForEachFixture(fixturePaths, fixture => evaluateFixture(fixture, projectASTCollection));
  });
  return _evaluateFixtures.apply(this, arguments);
}

function getOperationsForProject(_x3) {
  return _getOperationsForProject.apply(this, arguments);
}

function _getOperationsForProject() {
  _getOperationsForProject = _asyncToGenerator(function* (projectConfig) {
    const operationPaths = yield getGraphQLProjectIncludedFilePaths(projectConfig);
    const operationSources = yield Promise.all(operationPaths.map(loadOperationSource));
    const document = concatAST(operationSources.map(source => {
      try {
        return parse(source);
      } catch (error) {
        throw new Error(`Error parsing '${source.name}':\n\n${error.message.replace(/Syntax Error.*?\(.*?\) /, '')}`);
      }
    }));
    let schema;

    try {
      schema = yield projectConfig.getSchema();
    } catch (error) {
      throw new Error(`Error parsing '${projectConfig.schema}':\n\n${error.message.replace(/Syntax Error.*?\(.*?\) /, '')}`);
    }

    return {
      ast: compile(schema, document),
      config: projectConfig
    };
  });
  return _getOperationsForProject.apply(this, arguments);
}

function loadOperationSource(_x4) {
  return _loadOperationSource.apply(this, arguments);
}

function _loadOperationSource() {
  _loadOperationSource = _asyncToGenerator(function* (filePath) {
    const body = yield readFile(filePath, 'utf8');
    return new Source(body, filePath);
  });
  return _loadOperationSource.apply(this, arguments);
}

function runForEachFixture(fixturePaths, runner) {
  return Promise.all(fixturePaths.map( /*#__PURE__*/function () {
    var _ref = _asyncToGenerator(function* (fixturePath) {
      try {
        const fixture = yield readJSON(fixturePath);
        return _objectSpread2({
          fixturePath
        }, runner({
          path: fixturePath,
          content: fixture
        }));
      } catch (error) {
        return {
          fixturePath,
          scriptError: error,
          validationErrors: []
        };
      }
    });

    return function (_x5) {
      return _ref.apply(this, arguments);
    };
  }()));
}

function evaluateFixture(fixture, projectASTCollection) {
  try {
    const {
      operation,
      projectAST: {
        ast
      }
    } = getOperationForFixture(fixture, projectASTCollection);
    return validateFixture(fixture, ast, operation);
  } catch (error) {
    if (error instanceof MissingOperationError || error instanceof AmbiguousOperationNameError) {
      return {
        fixturePath: fixture.path,
        validationErrors: [error]
      };
    }

    throw error;
  }
}

export { evaluateFixtures };
